/// <reference path="../reference/cristalix.d.ts" />
export declare const CENTER: {
    x: number;
    y: number;
};
export declare const LEFT: {
    x: number;
    y: number;
};
export declare const RIGHT: {
    x: number;
    y: number;
};
export declare const TOP: {
    x: number;
    y: number;
};
export declare const BOTTOM: {
    x: number;
    y: number;
};
export declare const TOP_RIGHT: {
    x: number;
    y: number;
};
export declare const TOP_LEFT: {
    x: number;
    y: number;
};
export declare const BOTTOM_RIGHT: {
    x: number;
    y: number;
};
export declare const BOTTOM_LEFT: {
    x: number;
    y: number;
};
export declare type Callback = () => void;
export declare class Animation {
    readonly element: Element;
    readonly propertyId: number;
    startedTime: number;
    duration: number;
    startValue: number;
    targetValue: number;
    easingId: number;
    lastValue: number;
    constructor(element: Element, propertyId: number);
    newTarget(value: number, duration: number, easingId: number): void;
    update(time: number): boolean;
}
export declare type V2 = {
    x?: number;
    y?: number;
};
export declare type V3 = V2 & {
    z?: number;
};
export declare type Rotation = V3 & {
    angle: number;
};
export declare type ElementData = {
    offset?: V3;
    align?: V3;
    origin?: V3;
    scale?: V3;
    rotation?: Rotation;
    color?: Color;
    enabled?: boolean;
    beforeRender?: () => {};
    afterRender?: () => {};
    onClick?: (element: Element, button: number) => void;
    onHover?: (element: Element, hovered: boolean) => void;
};
export declare type RectData = {
    texture?: string | ResourceLocation;
    size?: V2;
    textureFrom?: V2;
    textureSize?: V2;
    children?: Element[];
} & ElementData;
export declare type Color = {
    a?: number;
    r?: number;
    g?: number;
    b?: number;
};
export declare function color2Hex(color: Color): number;
export declare function colorParts2Hex(a: number, r: number, g: number, b: number): number;
export declare function hex2Color(value: number, includeAlpha?: boolean): Color;
export declare function getTime(): number;
export declare type MouseHandler = (screenState: ScreenState) => void;
export declare type ItemData = {
    readonly item: ItemStack;
} & ElementData;
export declare type TextData = {
    readonly text?: string;
    readonly shadow?: boolean;
    readonly autoFit?: boolean;
} & ElementData;
export declare type ScreenState = {
    scaleFactor: number;
    screenWidth: number;
    screenHeight: number;
    unicodeFlag: boolean;
};
export declare class Task {
    time: number;
    action: () => void;
    constructor(time: number, action: () => void);
}
export declare function schedule(delay: number, action: () => void): void;
export declare function text(data: TextData): Text;
export declare function rect(data: RectData): Rect;
export declare abstract class Element {
    properties: number[];
    matrices: Matrix4f[];
    enabled: boolean;
    dirtyMatrices: number[];
    hovered: boolean;
    beforeRender?: () => void;
    afterRender?: () => void;
    passedHoverCulling: boolean;
    cachedHexColor: number;
    onHover?: (element: Element, hovered: boolean) => void;
    onClick?: (element: Element, button: number) => void;
    constructor(data: ElementData);
    applyTransformations(): void;
    setOffset(offset: V3, duration?: number, easingId?: number): void;
    setAlign(align: V3, duration?: number, easingId?: number): void;
    setOrigin(origin: V3, duration?: number, easingId?: number): void;
    setScale(scale: V3, duration?: number, easingId?: number): void;
    setRotation(rotation: Rotation, duration?: number, easingId?: number): void;
    setProperty(propertyId: number, value: number, duration?: number, easingId?: number): void;
    markDirty(matrixIds: number[]): void;
    cleanMatrices(): void;
    updateMatrix(matrixId: number): void;
    abstract render(): void;
}
export declare class Text extends Element {
    text?: string;
    autoFit: boolean;
    shadow: boolean;
    constructor(data: TextData);
    updateMatrix(matrixId: number): void;
    setText(text: string): void;
    render(): void;
}
export declare class Rect extends Element {
    children: Element[];
    texture: string | ResourceLocation;
    constructor(data: RectData);
    updateMatrix(matrixId: number): void;
    render(): void;
}
export declare class Item extends Element {
    item: ItemStack;
    constructor(data: ItemData);
    render(): void;
}
export declare function getScreenState(): ScreenState;
export declare const overlay: Element[];
export declare var zIndex: number;
