/// <reference path="../reference/cristalix.d.ts" />
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import * as index from './index';
var glPushMatrix = GL11.glPushMatrix;
var glPopMatrix = GL11.glPopMatrix;
var glMultMatrix = GL11.glMultMatrix;
var glColor = GL11.glColor4f;
export var CENTER = { x: 0.5, y: 0.5 };
export var LEFT = { x: 0, y: 0.5 };
export var RIGHT = { x: 1, y: 0.5 };
export var TOP = { x: 0.5, y: 0 };
export var BOTTOM = { x: 0.5, y: 1 };
export var TOP_RIGHT = { x: 1, y: 0 };
export var TOP_LEFT = { x: 0, y: 0 };
export var BOTTOM_RIGHT = { x: 1, y: 1 };
export var BOTTOM_LEFT = { x: 0, y: 1 };
var Animation = /** @class */ (function () {
    function Animation(element, propertyId) {
        this.element = element;
        this.propertyId = propertyId;
        this.startedTime = 0;
        this.duration = 0;
        this.startValue = 0;
        this.targetValue = 0;
        this.easingId = 0;
        this.targetValue = this.lastValue = element.properties[propertyId];
    }
    Animation.prototype.newTarget = function (value, duration, easingId) {
        this.startValue = this.lastValue;
        this.targetValue = value;
        this.startedTime = getTime();
        this.duration = duration;
        this.easingId = easingId;
    };
    Animation.prototype.update = function (time) {
        // if (!this.startedTime) return NaN;
        var part = (time - this.startedTime) / this.duration;
        var alive = part <= 1.0;
        var value;
        if (alive) {
            var easingId = this.easingId;
            if (easingId)
                part = NativeRuntime.ease(part, easingId);
            var startValue = this.startValue;
            value = startValue + (this.targetValue - startValue) * part;
        }
        else {
            this.startedTime = 0.0;
            value = this.targetValue;
        }
        this.lastValue = value;
        this.element.setProperty(this.propertyId, value);
        return alive;
    };
    return Animation;
}());
export { Animation };
export function color2Hex(color) {
    return colorParts2Hex(color.a, color.r, color.g, color.b);
}
export function colorParts2Hex(a, r, g, b) {
    return a * 255 << 24 |
        r * 255 << 16 |
        g * 255 << 8 |
        b * 255;
}
export function hex2Color(value, includeAlpha) {
    var a = includeAlpha ? ((value >> 24) & 0xFF) / 255.0 : 1;
    var r = ((value >> 16) & 0xFF) / 255.0;
    var g = ((value >> 8) & 0xFF) / 255.0;
    var b = (value & 0xFF) / 255.0;
    return { a: a, r: r, g: g, b: b };
}
export function getTime() {
    return System.currentTimeMillis();
}
var Task = /** @class */ (function () {
    function Task(time, action) {
        this.time = time;
        this.action = action;
    }
    return Task;
}());
export { Task };
export function schedule(delay, action) {
    runningTasks.push(new Task(getTime() + delay, action));
}
var runningTasks = [];
var runningAnimations = [];
function updateAnimations() {
    var time = getTime();
    var newRunningTasks = [];
    for (var _i = 0, runningTasks_1 = runningTasks; _i < runningTasks_1.length; _i++) {
        var task = runningTasks_1[_i];
        if (time >= task.time)
            task.action();
        else
            newRunningTasks.push(task);
    }
    runningTasks = newRunningTasks;
    var newRunningAnimations = [];
    for (var _a = 0, runningAnimations_1 = runningAnimations; _a < runningAnimations_1.length; _a++) {
        var animation = runningAnimations_1[_a];
        if (animation.update(time))
            newRunningAnimations.push(animation);
    }
    for (var _b = 0, runningAnimations_2 = runningAnimations; _b < runningAnimations_2.length; _b++) {
        var animation = runningAnimations_2[_b];
        animation.element.cleanMatrices();
    }
    runningAnimations = newRunningAnimations;
}
function hoverCulling(element) {
    var passed = false;
    if (element instanceof Rect) {
        for (var _i = 0, _a = element.children; _i < _a.length; _i++) {
            var child = _a[_i];
            if (hoverCulling(child)) {
                passed = true;
            }
        }
    }
    if (element.onHover || element.onClick) {
        passed = true;
    }
    element.passedHoverCulling = passed;
    return passed;
}
function updateHoverStates(element, baseMatrix, mouse) {
    if (!element.passedHoverCulling)
        return;
    var matrix = new Matrix4f();
    matrix.load(baseMatrix);
    for (var _i = 0, _a = element.matrices; _i < _a.length; _i++) {
        var m = _a[_i];
        if (m)
            Matrix4f.mul(matrix, m, matrix);
    }
    if (element.onHover || element.onClick) {
        var sizeX = element.properties[index.sizeX];
        var sizeY = element.properties[index.sizeY];
        var vector = new Vector4f(mouse.x, mouse.y, 0, 1);
        var inv = new Matrix4f();
        Matrix4f.invert(matrix, inv);
        Matrix4f.transform(inv, vector, vector);
        // testText.text = vector.getX() + ' ' + vector.getY();
        var x = vector.getX();
        var y = vector.getY();
        var hovered = x >= 0 && x < sizeX && y >= 0 && y < sizeY;
        if (element.hovered != hovered) {
            if (element.onHover)
                element.onHover(element, hovered);
            element.hovered = hovered;
        }
    }
    if (element instanceof Rect) {
        for (var _b = 0, _c = element.children; _b < _c.length; _b++) {
            var child = _c[_b];
            // updateHoverStates(child, new Vector4f(w, x, y, z));
            updateHoverStates(child, matrix, mouse);
        }
    }
}
export function text(data) {
    var element = new Text(data);
    element.cleanMatrices();
    return element;
}
export function rect(data) {
    var element = new Rect(data);
    element.cleanMatrices();
    return element;
}
var buffer = GLAllocation.createDirectFloatBuffer(16);
var Element = /** @class */ (function () {
    function Element(data) {
        var offset = data.offset, scale = data.scale, align = data.align, origin = data.origin, rotation = data.rotation, enabled = data.enabled, color = data.color;
        this.enabled = enabled == null ? true : enabled;
        var matrices = new Array(index.matrixFields);
        var properties = new Array(index.valueFields);
        // NativeRuntime.fillIntArray(properties, 0, properties.length, 0);
        for (var i = 0; i < properties.length; i++) {
            properties[i] = 0;
        }
        this.properties = properties;
        this.matrices = matrices;
        // properties.push(
        if (offset && offset.x)
            this.setProperty(index.offsetX, offset.x);
        if (offset && offset.y)
            this.setProperty(index.offsetY, offset.y);
        if (offset && offset.z)
            this.setProperty(index.offsetZ, offset.z);
        this.setProperty(index.scaleX, scale && scale.x ? scale.x : 1);
        this.setProperty(index.scaleY, scale && scale.y ? scale.y : 1);
        this.setProperty(index.scaleZ, scale && scale.z ? scale.z : 1);
        if (align && align.x)
            this.setProperty(index.alignX, align.x);
        if (align && align.y)
            this.setProperty(index.alignY, align.y);
        if (align && align.z)
            this.setProperty(index.alignZ, align.z);
        if (origin && origin.x)
            this.setProperty(index.originX, origin.x);
        if (origin && origin.y)
            this.setProperty(index.originY, origin.y);
        if (origin && origin.z)
            this.setProperty(index.originZ, origin.z);
        if (rotation && rotation.x)
            this.setProperty(index.rotationX, rotation.x);
        if (rotation && rotation.y)
            this.setProperty(index.rotationY, rotation.y);
        this.setProperty(index.rotationZ, rotation ? (rotation.z || 0) : 1);
        if (rotation && rotation.angle)
            this.setProperty(index.rotationAngle, rotation.angle);
        if (color) {
            this.setProperty(index.a, color.a == undefined ? 1 : color.a);
            this.setProperty(index.r, color.r == undefined ? 1 : color.r);
            this.setProperty(index.g, color.g == undefined ? 1 : color.g);
            this.setProperty(index.b, color.b == undefined ? 1 : color.b);
        }
        properties[index.parentSizeX] = 0;
        properties[index.parentSizeY] = 0;
        properties[index.parentSizeZ] = 0;
        // );
        this.onClick = data.onClick || null;
        this.onHover = data.onHover || null;
        this.beforeRender = data.beforeRender || null;
        this.afterRender = data.afterRender || null;
    }
    ;
    Element.prototype.applyTransformations = function () {
        if (!this.enabled)
            return;
        this.cleanMatrices();
        for (var i = 0; i < index.matrixFields; i++) {
            var matrix = this.matrices[i];
            if (matrix) {
                matrix.store(buffer);
                buffer.flip();
                glMultMatrix(buffer);
            }
        }
    };
    Element.prototype.setOffset = function (offset, duration, easingId) {
        var x = offset.x;
        var y = offset.y;
        var z = offset.z;
        if (x !== undefined)
            this.setProperty(index.offsetX, x, duration, easingId);
        if (y !== undefined)
            this.setProperty(index.offsetY, y, duration, easingId);
        if (z !== undefined)
            this.setProperty(index.offsetZ, z, duration, easingId);
    };
    Element.prototype.setAlign = function (align, duration, easingId) {
        var x = align.x;
        var y = align.y;
        var z = align.z;
        if (x !== undefined)
            this.setProperty(index.alignX, x, duration, easingId);
        if (y !== undefined)
            this.setProperty(index.alignY, y, duration, easingId);
        if (z !== undefined)
            this.setProperty(index.alignZ, z, duration, easingId);
    };
    Element.prototype.setOrigin = function (origin, duration, easingId) {
        var x = origin.x;
        var y = origin.y;
        var z = origin.z;
        if (x !== undefined)
            this.setProperty(index.originX, x, duration, easingId);
        if (y !== undefined)
            this.setProperty(index.originY, y, duration, easingId);
        if (z !== undefined)
            this.setProperty(index.originZ, z, duration, easingId);
    };
    Element.prototype.setScale = function (scale, duration, easingId) {
        var x = scale.x;
        var y = scale.y;
        var z = scale.z;
        if (x !== undefined)
            this.setProperty(index.scaleX, x, duration, easingId);
        if (y !== undefined)
            this.setProperty(index.scaleY, y, duration, easingId);
        if (z !== undefined)
            this.setProperty(index.scaleZ, z, duration, easingId);
    };
    Element.prototype.setRotation = function (rotation, duration, easingId) {
        var x = rotation.x;
        var y = rotation.y;
        var z = rotation.z;
        var angle = rotation.angle;
        if (angle !== undefined)
            this.setProperty(index.rotationAngle, angle, duration, easingId);
        if (x !== undefined)
            this.setProperty(index.rotationX, x, duration, easingId);
        if (y !== undefined)
            this.setProperty(index.rotationY, y, duration, easingId);
        if (z !== undefined)
            this.setProperty(index.rotationZ, z, duration, easingId);
    };
    Element.prototype.setProperty = function (propertyId, value, duration, easingId) {
        if (duration) {
            var animation = void 0;
            for (var _i = 0, runningAnimations_3 = runningAnimations; _i < runningAnimations_3.length; _i++) {
                var existing = runningAnimations_3[_i];
                if (existing.element === this && existing.propertyId === propertyId) {
                    animation = existing;
                    break;
                }
            }
            if (!animation)
                animation = new Animation(this, propertyId);
            animation.newTarget(value, duration, easingId);
            runningAnimations.push(animation);
            return;
        }
        // stdout.println('setting ' + propertyId + ' to ' + value);
        this.properties[propertyId] = value;
        var influence = index.matrixInfluence[propertyId];
        if (influence.length) {
            this.markDirty(influence);
        }
    };
    Element.prototype.markDirty = function (matrixIds) {
        if (!this.dirtyMatrices)
            this.dirtyMatrices = [];
        loop: for (var _i = 0, matrixIds_1 = matrixIds; _i < matrixIds_1.length; _i++) {
            var matrixId = matrixIds_1[_i];
            for (var _a = 0, _b = this.dirtyMatrices; _a < _b.length; _a++) {
                var dirtyMatrix = _b[_a];
                if (dirtyMatrix == matrixId)
                    continue loop;
            }
            // stdout.println('Marking ' + matrixId + ' as dirty');
            this.dirtyMatrices.push(matrixId);
        }
    };
    Element.prototype.cleanMatrices = function () {
        var dirty = this.dirtyMatrices;
        if (!dirty || !dirty.length)
            return;
        for (var _i = 0, dirty_1 = dirty; _i < dirty_1.length; _i++) {
            var dirtyMatrixId = dirty_1[_i];
            if (dirtyMatrixId == -128)
                continue;
            this.updateMatrix(dirtyMatrixId);
        }
        this.dirtyMatrices = null;
    };
    Element.prototype.updateMatrix = function (matrixId) {
        // stdout.println('updating matrix ' + matrixId)
        var properties = this.properties;
        if (this.dirtyMatrices) {
            this.dirtyMatrices[this.dirtyMatrices.indexOf(matrixId)] = -128;
        }
        if (matrixId == index.colorMatrix) {
            this.cachedHexColor = colorParts2Hex(properties[index.a], properties[index.r], properties[index.g], properties[index.b]);
        }
        if (matrixId >= 0) {
            var matrix = new Matrix4f().setIdentity();
            switch (matrixId) {
                case index.alignMatrix:
                    matrix.translate(new Vector3f(properties[index.alignX] * properties[index.parentSizeX], properties[index.alignY] * properties[index.parentSizeY], properties[index.alignZ] * properties[index.parentSizeZ]));
                    break;
                case index.rotationMatrix:
                    matrix.rotate(properties[index.rotationAngle], new Vector3f(properties[index.rotationX], properties[index.rotationY], properties[index.rotationZ]));
                    break;
                case index.offsetMatrix:
                    matrix.translate(new Vector3f(properties[index.offsetX], properties[index.offsetY], properties[index.offsetZ]));
                    break;
                case index.scaleMatrix:
                    matrix.scale(new Vector3f(properties[index.scaleX], properties[index.scaleY], properties[index.scaleZ]));
                    break;
                case index.originMatrix:
                    matrix.translate(new Vector3f(-properties[index.originX] * properties[index.sizeX], -properties[index.originY] * properties[index.sizeY], -properties[index.originZ] * properties[index.sizeZ]));
                    break;
            }
            this.matrices[matrixId] = matrix;
        }
    };
    return Element;
}());
export { Element };
var Text = /** @class */ (function (_super) {
    __extends(Text, _super);
    function Text(data) {
        var _this = _super.call(this, data) || this;
        _this.text = data.text || "";
        _this.shadow = !!data.shadow;
        _this.autoFit = !!data.autoFit;
        _this.setProperty(index.sizeX, fontRenderer.getStringWidth(_this.text));
        _this.properties[index.sizeY] = 9;
        return _this;
    }
    Text.prototype.updateMatrix = function (matrixId) {
        if ((matrixId == index.sizeMatrix || matrixId == index.scaleMatrix) && this.autoFit) {
            var textWidth = this.properties[index.sizeX];
            var parentWidth = this.properties[index.parentSizeX];
            if (textWidth) {
                var factor = (parentWidth - 2) / textWidth;
                if (factor < 1)
                    this.properties[index.scaleX] = factor;
            }
        }
        _super.prototype.updateMatrix.call(this, matrixId);
    };
    Text.prototype.setText = function (text) {
        this.text = text;
        this.setProperty(index.sizeX, Draw.getStringWidth(text));
    };
    Text.prototype.render = function () {
        if (!this.enabled)
            return;
        glPushMatrix();
        _super.prototype.applyTransformations.call(this);
        // GL11.glDepthFunc(GL11.GL_LESS);
        // GlStateManager.disableDepth();
        // ToDo: colored text
        fontRenderer.drawString(this.text, 0, fontRenderer.getUnicodeFlag() ? 0 : 1, -1, this.shadow);
        // GL11.glDepthFunc(GL11.GL_LEQUAL);
        // GlStateManager.enableDepth();
        glPopMatrix();
    };
    return Text;
}(Element));
export { Text };
var Rect = /** @class */ (function (_super) {
    __extends(Rect, _super);
    function Rect(data) {
        var _this = _super.call(this, data) || this;
        if (data.size) {
            if (data.size.x)
                _this.setProperty(index.sizeX, data.size.x);
            if (data.size.y)
                _this.setProperty(index.sizeY, data.size.y);
        }
        if (data.textureFrom) {
            if (data.textureFrom.x)
                _this.setProperty(index.textureX, data.textureFrom.x);
            if (data.textureFrom.y)
                _this.setProperty(index.textureY, data.textureFrom.y);
        }
        _this.setProperty(index.textureWidth, data.textureSize ? data.textureSize.x || 1 : 1);
        _this.setProperty(index.textureHeight, data.textureSize ? data.textureSize.y || 1 : 1);
        _this.children = data.children || [];
        _this.texture = data.texture || null;
        return _this;
    }
    Rect.prototype.updateMatrix = function (matrixId) {
        if (matrixId === index.sizeMatrix) {
            var properties = this.properties;
            for (var _i = 0, _a = this.children; _i < _a.length; _i++) {
                var child = _a[_i];
                var childProperties = child.properties;
                childProperties[index.parentSizeX] = properties[index.sizeX];
                childProperties[index.parentSizeY] = properties[index.sizeY];
                childProperties[index.parentSizeZ] = properties[index.sizeZ];
                child.updateMatrix(index.alignMatrix);
            }
        }
        _super.prototype.updateMatrix.call(this, matrixId);
    };
    Rect.prototype.render = function () {
        if (!this.enabled)
            return;
        glPushMatrix();
        _super.prototype.applyTransformations.call(this);
        if (this.beforeRender)
            this.beforeRender();
        var properties = this.properties;
        if (this.texture) {
            GlStateManager.enableBlend();
            Textures.bindTexture(this.texture);
            var precision = 1073741824;
            Draw.drawScaledCustomSizeModalRect(0, 0, properties[index.textureX] * precision, properties[index.textureY] * precision, properties[index.textureWidth] * precision, properties[index.textureHeight] * precision, properties[index.sizeX], properties[index.sizeY], precision, precision);
        }
        else {
            Draw.drawRect(0, 0, properties[index.sizeX], properties[index.sizeY], this.cachedHexColor);
        }
        if (this.afterRender)
            this.afterRender();
        // translate(0, 0, 0.01);
        for (var i = 0; i < this.children.length; i++) {
            this.children[i].render();
        }
        glPopMatrix();
    };
    return Rect;
}(Element));
export { Rect };
var Item = /** @class */ (function (_super) {
    __extends(Item, _super);
    function Item(data) {
        var _this = _super.call(this, data) || this;
        _this.item = data.item;
        _this.properties.push(16, 16, 0); // size
        return _this;
    }
    Item.prototype.render = function () {
        if (!this.enabled)
            return;
        glPushMatrix();
        _super.prototype.applyTransformations.call(this);
        RenderHelper.enableGUIStandardItemLighting();
        Draw.renderItemAndEffectIntoGUI(this.item, 0, 0);
        RenderHelper.disableStandardItemLighting();
        glPopMatrix();
    };
    return Item;
}(Element));
export { Item };
export function getScreenState() {
    var resolution = minecraft.getResolution();
    var scaleFactor = resolution.getScaleFactor();
    var screenWidth = resolution.getScaledWidth();
    var screenHeight = resolution.getScaledHeight();
    var unicodeFlag = fontRenderer.getUnicodeFlag();
    return {
        scaleFactor: scaleFactor,
        screenWidth: screenWidth,
        screenHeight: screenHeight,
        unicodeFlag: unicodeFlag
    };
}
var lastScreenState = getScreenState();
// let mouseX = Mouse.getX() / factor;
// let mouseY = screenHeight - Mouse.getY() / factor;
// let time = getTime();
export var overlay = [];
export var zIndex = 0;
Events.on(plugin, 'gui_overlay_render', function (e) {
    var screenState = getScreenState();
    if (lastScreenState.scaleFactor != screenState.scaleFactor ||
        lastScreenState.screenWidth != screenState.screenWidth ||
        lastScreenState.screenHeight != screenState.screenHeight ||
        lastScreenState.unicodeFlag != screenState.unicodeFlag) {
        for (var _i = 0, overlay_1 = overlay; _i < overlay_1.length; _i++) {
            var element = overlay_1[_i];
            element.setProperty(index.parentSizeX, screenState.screenWidth);
            element.setProperty(index.parentSizeY, screenState.screenHeight);
        }
        lastScreenState = screenState;
    }
    updateAnimations();
    for (var _a = 0, overlay_2 = overlay; _a < overlay_2.length; _a++) {
        var element = overlay_2[_a];
        hoverCulling(element);
        // updateHoverStates(element, new Vector4f(Mouse.getX() / screenState.scaleFactor, (Display.getHeight() - Mouse.getY()) / screenState.scaleFactor, 0, 1));
        updateHoverStates(element, new Matrix4f().setIdentity(), {
            x: Mouse.getX() / screenState.scaleFactor,
            y: (Display.getHeight() - Mouse.getY()) / screenState.scaleFactor
        });
        element.render();
    }
});
// let testText = text({text: 'hello world'});
// let test = rect({
//     size: {x: 100, y: 20},
//     color: {a: 1, r: 1, g: 0.5, b: 0.1},
//     offset: {x: 30, y: 10},
//     rotation: {angle: -100, x: 0, y: 0, z: 1},
//     children: [
//         testText
//     ],
//     onHover: (elem, hovered) => {
//         elem.setProperty(index.r, hovered ? 1 : 0);
//     }
// });
// overlay.push(test);
// Events.on(plugin, 'key_press', (e: KeyPressEvent) => {
//     if (e.key == Keyboard.KEY_J) {
//         test.setProperty(index.offsetX, Math.random() * 500, 1000, 0);
//     }
// })
// ToDo: one-time click detection
// const mouseButtons = [
//     {event: 'left_click', pressed: false, property: 'onLeftClick'},
//     {event: 'right_click', pressed: false, property: 'onRightClick'},
// ]
Events.on(plugin, 'game_loop', function (e) {
    var screenState = getScreenState();
    // for (let element of overlay) {
    //     element.checkHovered(screenState, screenState.width, screenState.height);
    // }
    // for (var i = 0; i < mouseButtons.length; i++) {
    //     let button = mouseButtons[i];
    //     if (!Mouse.isButtonDown(i)) {
    //         button.pressed = false;
    //         continue;
    //     }
    //     if (button.pressed) continue;
    //     button.pressed = true;
    //     for (var j = 0; j < root.children.length; j++) {
    //         if (root.children[j].executeHovered) 
    //             root.children[j].executeHovered(screenState, button.property, screenState.width, screenState.height);
    //     }
    //     Events.post(button.event, screenState);
    // }
});
